@@ -9,8 +9,8 @@ from xonsh.tools import ON_WINDOWS
 
 import pytest
 
-from xonsh.environ import (Env, load_static_config,
-    locate_binary, partial_format_prompt)
+from xonsh.commands_cache import CommandsCache
+from xonsh.environ import Env, load_static_config, locate_binary
 
 from tools import skip_if_on_unix
 
@@ -55,64 +55,6 @@ def test_env_detype_no_dict():
     det = env.detype()
     assert 'YO' not in det
 
-@pytest.mark.parametrize('formatter_dict',[{
-    'a_string': 'cat',
-    'none': (lambda: None),
-    'f': (lambda: 'wakka'),
-}])
-@pytest.mark.parametrize('inp, exp', [
-    ('my {a_string}', 'my cat'),
-    ('my {none}{a_string}', 'my cat'),
-    ('{f} jawaka', 'wakka jawaka'),
-])
-def test_format_prompt(inp, exp, formatter_dict):
-    obs = partial_format_prompt(template=inp, formatter_dict=formatter_dict)
-    assert exp == obs
-
-
-@pytest.mark.parametrize('formatter_dict',[{
-    'a_string': 'cats',
-    'a_number': 7,
-    'empty': '',
-    'current_job': (lambda: 'sleep'),
-    'none': (lambda: None),
-}])
-@pytest.mark.parametrize('inp, exp', [
-    ('{a_number:{0:^3}}cats', ' 7 cats'),
-    ( '{current_job:{} | }xonsh', 'sleep | xonsh'),
-    ( '{none:{} | }{a_string}{empty:!}', 'cats!'),
-    ( '{none:{}}', ''),
-    ( '{{{a_string:{{{}}}}}}', '{{cats}}'),
-    ( '{{{none:{{{}}}}}}', '{}'),
-])
-def test_format_prompt_with_format_spec(inp, exp, formatter_dict):
-    obs = partial_format_prompt(template=inp, formatter_dict=formatter_dict)
-    assert exp == obs
-
-def test_format_prompt_with_broken_template():
-    for p in ('{user', '{user}{hostname'):
-        assert partial_format_prompt(p) == p
-
-    # '{{user' will be parsed to '{user'
-    for p in ('{{user}', '{{user'):
-        assert 'user' in partial_format_prompt(p)
-
-def test_format_prompt_with_broken_template_in_func():
-    for p in (
-        lambda: '{user',
-        lambda: '{{user',
-        lambda: '{{user}',
-        lambda: '{user}{hostname',
-    ):
-        # '{{user' will be parsed to '{user'
-        assert 'user' in partial_format_prompt(p)
-
-def test_format_prompt_with_invalid_func():
-    def p():
-        foo = bar  # raises exception
-        return '{user}'
-    assert isinstance(partial_format_prompt(p), str)
-
 def test_histcontrol_none():
     env = Env(HISTCONTROL=None)
     assert isinstance(env['HISTCONTROL'], set)
@@ -192,8 +134,9 @@ def test_locate_binary_on_windows(xonsh_builtins):
             'PATH': [tmpdir],
             'PATHEXT': ['.COM', '.EXE', '.BAT'],
         })
-        assert locate_binary('file1') == os.path.join(tmpdir,'file1.exe')
-        assert locate_binary('file1.exe') == os.path.join(tmpdir,'file1.exe')
-        assert locate_binary('file2') == os.path.join(tmpdir,'FILE2.BAT')
-        assert locate_binary('file2.bat') == os.path.join(tmpdir,'FILE2.BAT')
+        xonsh_builtins.__xonsh_commands_cache__ = CommandsCache()
+        assert locate_binary('file1') == os.path.join(tmpdir, 'file1.exe')
+        assert locate_binary('file1.exe') == os.path.join(tmpdir, 'file1.exe')
+        assert locate_binary('file2') == os.path.join(tmpdir, 'FILE2.BAT')
+        assert locate_binary('file2.bat') == os.path.join(tmpdir, 'FILE2.BAT')
         assert locate_binary('file3') is None
