@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-"""Tests the xonsh history."""
+"""Tests the json history backend."""
 # pylint: disable=protected-access
 import io
 import os
@@ -7,15 +7,16 @@ import sys
 import shlex
 
 from xonsh.lazyjson import LazyJSON
-from xonsh.history import History, _hist_create_parser, _hist_parse_args
-from xonsh import history
+from xonsh.history.json import JsonHistory
+from xonsh.history.main import history_main, _xh_parse_args
 
 import pytest
 
 
 @pytest.yield_fixture
 def hist():
-    h = History(filename='xonsh-HISTORY-TEST.json', here='yup', sessionid='SESSIONID', gc=False)
+    h = JsonHistory(filename='xonsh-HISTORY-TEST.json', here='yup',
+                    sessionid='SESSIONID', gc=False)
     yield h
     os.remove(h.filename)
 
@@ -30,9 +31,19 @@ def test_hist_init(hist):
 def test_hist_append(hist, xonsh_builtins):
     """Verify appending to the history works."""
     xonsh_builtins.__xonsh_env__['HISTCONTROL'] = set()
-    hf = hist.append({'joco': 'still alive'})
+    hf = hist.append({'inp': 'still alive', 'rtn': 0})
     assert hf is None
-    assert 'still alive' == hist.buffer[0]['joco']
+    assert 'still alive' == hist.buffer[0]['inp']
+    assert 0 == hist.buffer[0]['rtn']
+    assert 0 == hist.rtns[-1]
+    hf = hist.append({'inp': 'dead now', 'rtn': 1})
+    assert 'dead now' == hist.buffer[1]['inp']
+    assert 1 == hist.buffer[1]['rtn']
+    assert 1 == hist.rtns[-1]
+    hf = hist.append({'inp': 'reborn', 'rtn': 0})
+    assert 'reborn' == hist.buffer[2]['inp']
+    assert 0 == hist.buffer[2]['rtn']
+    assert 0 == hist.rtns[-1]
 
 
 def test_hist_flush(hist, xonsh_builtins):
@@ -40,20 +51,61 @@ def test_hist_flush(hist, xonsh_builtins):
     hf = hist.flush()
     assert hf is None
     xonsh_builtins.__xonsh_env__['HISTCONTROL'] = set()
-    hist.append({'joco': 'still alive'})
+    hist.append({'inp': 'still alive?', 'rtn': 0, 'out': 'yes'})
     hf = hist.flush()
     assert hf is not None
     while hf.is_alive():
         pass
     with LazyJSON(hist.filename) as lj:
-        obs = lj['cmds'][0]['joco']
-    assert 'still alive' == obs
+        assert len(lj['cmds']) == 1
+        cmd = lj['cmds'][0]
+        assert cmd['inp'] == 'still alive?'
+        assert not cmd.get('out', None)
+
+
+def test_hist_flush_with_store_stdout(hist, xonsh_builtins):
+    """Verify explicit flushing of the history works."""
+    hf = hist.flush()
+    assert hf is None
+    xonsh_builtins.__xonsh_env__['HISTCONTROL'] = set()
+    xonsh_builtins.__xonsh_env__['XONSH_STORE_STDOUT'] = True
+    hist.append({'inp': 'still alive?', 'rtn': 0, 'out': 'yes'})
+    hf = hist.flush()
+    assert hf is not None
+    while hf.is_alive():
+        pass
+    with LazyJSON(hist.filename) as lj:
+        assert len(lj['cmds']) == 1
+        assert lj['cmds'][0]['inp'] == 'still alive?'
+        assert lj['cmds'][0]['out'].strip() == 'yes'
+
+
+def test_hist_flush_with_hist_control(hist, xonsh_builtins):
+    """Verify explicit flushing of the history works."""
+    hf = hist.flush()
+    assert hf is None
+    xonsh_builtins.__xonsh_env__['HISTCONTROL'] = 'ignoredups,ignoreerr'
+    hist.append({'inp': 'ls foo1', 'rtn': 0})
+    hist.append({'inp': 'ls foo1', 'rtn': 1})
+    hist.append({'inp': 'ls foo1', 'rtn': 0})
+    hist.append({'inp': 'ls foo2', 'rtn': 2})
+    hist.append({'inp': 'ls foo3', 'rtn': 0})
+    hf = hist.flush()
+    assert hf is not None
+    while hf.is_alive():
+        pass
+    assert len(hist.buffer) == 0
+    with LazyJSON(hist.filename) as lj:
+        cmds = list(lj['cmds'])
+        assert len(cmds) == 2
+        assert [x['inp'] for x in cmds] == ['ls foo1', 'ls foo3']
+        assert [x['rtn'] for x in cmds] == [0, 0]
 
 
 def test_cmd_field(hist, xonsh_builtins):
     # in-memory
     xonsh_builtins.__xonsh_env__['HISTCONTROL'] = set()
-    hf = hist.append({'rtn': 1})
+    hf = hist.append({'inp': 'ls foo', 'rtn': 1})
     assert hf is None
     assert 1 == hist.rtns[0]
     assert 1 == hist.rtns[-1]
@@ -85,14 +137,14 @@ def test_show_cmd_numerate(inp, commands, offset, hist, xonsh_builtins, capsys):
     base_idx, step = offset
     xonsh_builtins.__xonsh_history__ = hist
     xonsh_builtins.__xonsh_env__['HISTCONTROL'] = set()
-    for ts,cmd in enumerate(CMDS):  # populate the shell history
-        hist.append({'inp': cmd, 'rtn': 0, 'ts':(ts+1, ts+1.5)})
+    for ts, cmd in enumerate(CMDS):  # populate the shell history
+        hist.append({'inp': cmd, 'rtn': 0, 'ts':(ts + 1, ts + 1.5)})
 
     exp = ('{}: {}'.format(base_idx + idx * step, cmd)
            for idx, cmd in enumerate(list(commands)))
     exp = '\n'.join(exp)
 
-    history.history_main(['show', '-n'] + shlex.split(inp))
+    history_main(['show', '-n'] + shlex.split(inp))
     out, err = capsys.readouterr()
     assert out.rstrip() == exp
 
@@ -105,59 +157,77 @@ def test_histcontrol(hist, xonsh_builtins):
 
     # An error, buffer remains empty
     hist.append({'inp': 'ls foo', 'rtn': 2})
-    assert len(hist.buffer) == 0
+    assert len(hist.buffer) == 1
+    assert hist.rtns[-1] == 2
+    assert hist.inps[-1] == 'ls foo'
 
     # Success
     hist.append({'inp': 'ls foobazz', 'rtn': 0})
-    assert len(hist.buffer) == 1
+    assert len(hist.buffer) == 2
     assert 'ls foobazz' == hist.buffer[-1]['inp']
     assert 0 == hist.buffer[-1]['rtn']
+    assert hist.rtns[-1] == 0
+    assert hist.inps[-1] == 'ls foobazz'
 
     # Error
     hist.append({'inp': 'ls foo', 'rtn': 2})
-    assert len(hist.buffer) == 1
-    assert 'ls foobazz' == hist.buffer[-1]['inp']
-    assert 0 == hist.buffer[-1]['rtn']
+    assert len(hist.buffer) == 3
+    assert 'ls foo' == hist.buffer[-1]['inp']
+    assert 2 == hist.buffer[-1]['rtn']
+    assert hist.rtns[-1] == 2
+    assert hist.inps[-1] == 'ls foo'
 
     # File now exists, success
     hist.append({'inp': 'ls foo', 'rtn': 0})
-    assert len(hist.buffer) == 2
+    assert len(hist.buffer) == 4
     assert 'ls foo' == hist.buffer[-1]['inp']
     assert 0 == hist.buffer[-1]['rtn']
+    assert hist.rtns[-1] == 0
+    assert hist.inps[-1] == 'ls foo'
 
     # Success
     hist.append({'inp': 'ls', 'rtn': 0})
-    assert len(hist.buffer) == 3
+    assert len(hist.buffer) == 5
     assert 'ls' == hist.buffer[-1]['inp']
     assert 0 == hist.buffer[-1]['rtn']
+    assert hist.rtns[-1] == 0
+    assert hist.inps[-1] == 'ls'
 
     # Dup
     hist.append({'inp': 'ls', 'rtn': 0})
-    assert len(hist.buffer) == 3
+    assert len(hist.buffer) == 6
+    assert hist.rtns[-1] == 0
+    assert hist.inps[-1] == 'ls'
 
     # Success
     hist.append({'inp': '/bin/ls', 'rtn': 0})
-    assert len(hist.buffer) == 4
+    assert len(hist.buffer) == 7
     assert '/bin/ls' == hist.buffer[-1]['inp']
     assert 0 == hist.buffer[-1]['rtn']
+    assert hist.rtns[-1] == 0
+    assert hist.inps[-1] == '/bin/ls'
 
     # Error
     hist.append({'inp': 'ls bazz', 'rtn': 1})
-    assert len(hist.buffer) == 4
-    assert '/bin/ls' == hist.buffer[-1]['inp']
-    assert 0 == hist.buffer[-1]['rtn']
+    assert len(hist.buffer) == 8
+    assert 'ls bazz' == hist.buffer[-1]['inp']
+    assert 1 == hist.buffer[-1]['rtn']
+    assert hist.rtns[-1] == 1
+    assert hist.inps[-1] == 'ls bazz'
 
     # Error
     hist.append({'inp': 'ls bazz', 'rtn': -1})
-    assert len(hist.buffer) == 4
-    assert '/bin/ls' == hist.buffer[-1]['inp']
-    assert 0 == hist.buffer[-1]['rtn']
+    assert len(hist.buffer) == 9
+    assert 'ls bazz' == hist.buffer[-1]['inp']
+    assert -1 == hist.buffer[-1]['rtn']
+    assert hist.rtns[-1] == -1
+    assert hist.inps[-1] == 'ls bazz'
 
 
 @pytest.mark.parametrize('args', [ '-h', '--help', 'show -h', 'show --help'])
 def test_parse_args_help(args, capsys):
     with pytest.raises(SystemExit):
-        args = _hist_parse_args(shlex.split(args))
+        args = _xh_parse_args(shlex.split(args))
     assert 'show this help message and exit' in capsys.readouterr()[0]
 
 
@@ -183,7 +253,7 @@ def test_parser_show(args, exp):
               'end_time': None,
               'datetime_format': None,
               'timestamp': False}
-    ns = _hist_parse_args(shlex.split(args))
+    ns = _xh_parse_args(shlex.split(args))
     assert ns.__dict__ == exp_ns
 
 
