@@ -7,6 +7,7 @@ import ast
 import builtins
 import platform
 import subprocess
+import contextlib
 from collections import defaultdict
 from collections.abc import MutableMapping
 
@@ -24,6 +25,7 @@ ON_DARWIN = (platform.system() == 'Darwin')
 ON_WINDOWS = (platform.system() == 'Windows')
 ON_CONDA = True in [conda in pytest.__file__ for conda
                     in ['anaconda', 'miniconda']]
+TEST_DIR = os.path.dirname(__file__)
 
 # pytest skip decorators
 skip_if_py34 = pytest.mark.skipif(VER_MAJOR_MINOR < VER_3_5, reason="Py3.5+ only test")
@@ -85,8 +87,13 @@ class DummyHistory:
 
 class DummyEnv(MutableMapping):
 
+    DEFAULTS = {
+        'XONSH_DEBUG': 1,
+    }
+
     def __init__(self, *args, **kwargs):
-        self._d = dict(*args, **kwargs)
+        self._d = self.DEFAULTS.copy()
+        self._d.update(dict(*args, **kwargs))
 
     def detype(self):
         return {k: str(v) for k, v in self._d.items()}
@@ -106,6 +113,27 @@ class DummyEnv(MutableMapping):
     def __iter__(self):
         yield from self._d
 
+    @contextlib.contextmanager
+    def swap(self, other=None, **kwargs):
+        old = {}
+        # single positional argument should be a dict-like object
+        if other is not None:
+            for k, v in other.items():
+                old[k] = self.get(k, NotImplemented)
+                self[k] = v
+        # kwargs could also have been sent in
+        for k, v in kwargs.items():
+            old[k] = self.get(k, NotImplemented)
+            self[k] = v
+        yield self
+        # restore the values
+        for k, v in old.items():
+            if v is NotImplemented:
+                del self[k]
+            else:
+                self[k] = v
+
+
 #
 # Execer tools
 #
