@@ -1,7 +1,7 @@
-__version__ = '0.4.7'
+__version__ = '0.5.3'
 
 # amalgamate exclude jupyter_kernel parser_table parser_test_table pyghooks
-# amalgamate exclude winutils wizard pytest_plugin
+# amalgamate exclude winutils wizard pytest_plugin fs
 import os as _os
 if _os.getenv('XONSH_DEBUG', ''):
     pass
@@ -11,6 +11,8 @@ else:
         from xonsh import __amalgam__
         completer = __amalgam__
         _sys.modules['xonsh.completer'] = __amalgam__
+        contexts = __amalgam__
+        _sys.modules['xonsh.contexts'] = __amalgam__
         lazyasd = __amalgam__
         _sys.modules['xonsh.lazyasd'] = __amalgam__
         lazyjson = __amalgam__
@@ -19,8 +21,6 @@ else:
         _sys.modules['xonsh.ansi_colors'] = __amalgam__
         codecache = __amalgam__
         _sys.modules['xonsh.codecache'] = __amalgam__
-        lazyimps = __amalgam__
-        _sys.modules['xonsh.lazyimps'] = __amalgam__
         platform = __amalgam__
         _sys.modules['xonsh.platform'] = __amalgam__
         pretty = __amalgam__
@@ -29,10 +29,12 @@ else:
         _sys.modules['xonsh.timings'] = __amalgam__
         jobs = __amalgam__
         _sys.modules['xonsh.jobs'] = __amalgam__
+        lazyimps = __amalgam__
+        _sys.modules['xonsh.lazyimps'] = __amalgam__
         parser = __amalgam__
         _sys.modules['xonsh.parser'] = __amalgam__
-        teepty = __amalgam__
-        _sys.modules['xonsh.teepty'] = __amalgam__
+        style_tools = __amalgam__
+        _sys.modules['xonsh.style_tools'] = __amalgam__
         tokenize = __amalgam__
         _sys.modules['xonsh.tokenize'] = __amalgam__
         tools = __amalgam__
@@ -41,8 +43,6 @@ else:
         _sys.modules['xonsh.ast'] = __amalgam__
         commands_cache = __amalgam__
         _sys.modules['xonsh.commands_cache'] = __amalgam__
-        contexts = __amalgam__
-        _sys.modules['xonsh.contexts'] = __amalgam__
         diff_history = __amalgam__
         _sys.modules['xonsh.diff_history'] = __amalgam__
         events = __amalgam__
@@ -57,20 +57,14 @@ else:
         _sys.modules['xonsh.proc'] = __amalgam__
         xontribs = __amalgam__
         _sys.modules['xonsh.xontribs'] = __amalgam__
-        base_shell = __amalgam__
-        _sys.modules['xonsh.base_shell'] = __amalgam__
         dirstack = __amalgam__
         _sys.modules['xonsh.dirstack'] = __amalgam__
-        history = __amalgam__
-        _sys.modules['xonsh.history'] = __amalgam__
         inspectors = __amalgam__
         _sys.modules['xonsh.inspectors'] = __amalgam__
         xonfig = __amalgam__
         _sys.modules['xonsh.xonfig'] = __amalgam__
         environ = __amalgam__
         _sys.modules['xonsh.environ'] = __amalgam__
-        readline_shell = __amalgam__
-        _sys.modules['xonsh.readline_shell'] = __amalgam__
         tracer = __amalgam__
         _sys.modules['xonsh.tracer'] = __amalgam__
         replay = __amalgam__
@@ -85,8 +79,12 @@ else:
         _sys.modules['xonsh.imphooks'] = __amalgam__
         shell = __amalgam__
         _sys.modules['xonsh.shell'] = __amalgam__
+        base_shell = __amalgam__
+        _sys.modules['xonsh.base_shell'] = __amalgam__
         main = __amalgam__
         _sys.modules['xonsh.main'] = __amalgam__
+        readline_shell = __amalgam__
+        _sys.modules['xonsh.readline_shell'] = __amalgam__
         del __amalgam__
     except ImportError:
         pass
