@@ -108,6 +108,30 @@ def gather_names(node):
     return rtn
 
 
+def get_id_ctx(node):
+    """Gets the id and attribute of a node, or returns a default."""
+    nid = getattr(node, 'id', None)
+    if nid is None:
+        return (None, None)
+    return (nid, node.ctx)
+
+
+def gather_load_store_names(node):
+    """Returns the names present in the node's tree in a set of load nodes and
+    a set of store nodes.
+    """
+    load = set()
+    store = set()
+    for nid, ctx in map(get_id_ctx, walk(node)):
+        if nid is None:
+            continue
+        elif isinstance(ctx, Load):
+            load.add(nid)
+        else:
+            store.add(nid)
+    return (load, store)
+
+
 def has_elts(x):
     """Tests if x is an AST node with elements."""
     return isinstance(x, AST) and hasattr(x, 'elts')
@@ -226,95 +250,18 @@ class CtxAwareTransformer(NodeTransformer):
 
     def is_in_scope(self, node):
         """Determines whether or not the current node is in scope."""
-        lname = leftmostname(node)
-        if lname is None:
-            return node
+        names, store = gather_load_store_names(node)
+        names -= store
+        if not names:
+            return True
         inscope = False
         for ctx in reversed(self.contexts):
-            if lname in ctx:
+            names -= ctx
+            if not names:
                 inscope = True
                 break
         return inscope
 
-    #
-    # With Transformers
-    #
-    def insert_with_block_check(self, node):
-        """Modifies a with statement node in-place to add an initial check
-        for whether or not the block should be executed. If the block is
-        not executed it will raise a XonshBlockError containing the required
-        information.
-        """
-        nwith = self._nwith  # the nesting level of the current with-statement
-        lineno = get_lineno(node)
-        col = get_col(node, 0)
-        # Add or discover target names
-        targets = set()
-        i = 0  # index of unassigned items
-
-        def make_next_target():
-            nonlocal i
-            targ = '__xonsh_with_target_{}_{}__'.format(nwith, i)
-            n = Name(id=targ, ctx=Store(), lineno=lineno, col_offset=col)
-            targets.add(targ)
-            i += 1
-            return n
-        for item in node.items:
-            if item.optional_vars is None:
-                if has_elts(item.context_expr):
-                    targs = [make_next_target() for _ in item.context_expr.elts]
-                    optvars = Tuple(elts=targs, ctx=Store(), lineno=lineno,
-                                    col_offset=col)
-                else:
-                    optvars = make_next_target()
-                item.optional_vars = optvars
-            else:
-                targets.update(gather_names(item.optional_vars))
-        # Ok, now that targets have been found / created, make the actual check
-        # to see if we are in a non-executing block. This is equivalent to
-        # writing the following condition:
-        #
-        #     if getattr(targ0, '__xonsh_block__', False) or \
-        #        getattr(targ1, '__xonsh_block__', False) or ...:
-        #         raise XonshBlockError(lines, globals(), locals())
-        tests = [_getblockattr(t, lineno, col) for t in sorted(targets)]
-        if len(tests) == 1:
-            test = tests[0]
-        else:
-            test = BoolOp(op=Or(), values=tests, lineno=lineno, col_offset=col)
-        ldx, udx = self._find_with_block_line_idx(node)
-        lines = [Str(s=s, lineno=lineno, col_offset=col)
-                 for s in self.lines[ldx:udx]]
-        check = If(test=test, body=[
-            Raise(exc=xonsh_call('XonshBlockError',
-                                 args=[List(elts=lines, ctx=Load(),
-                                            lineno=lineno, col_offset=col),
-                                       xonsh_call('globals', args=[],
-                                                  lineno=lineno, col=col),
-                                       xonsh_call('locals', args=[],
-                                                  lineno=lineno, col=col)],
-                                 lineno=lineno, col=col),
-                  cause=None, lineno=lineno, col_offset=col)],
-                orelse=[], lineno=lineno, col_offset=col)
-        node.body.insert(0, check)
-
-    def _find_with_block_line_idx(self, node):
-        ldx = min_line(node.body[0]) - 1
-        udx = max_line(node.body[-1])
-        # now check if parsable, or add lines until it is or we run out of lines
-        nlines = len(self.lines)
-        lines = 'with __xonsh_dummy__:\n' + '\n'.join(self.lines[ldx:udx])
-        lines += '\n'
-        parsable = False
-        while not parsable and udx < nlines:
-            try:
-                self.parser.parse(lines, mode=self.mode)
-                parsable = True
-            except SyntaxError:
-                lines += self.lines[udx] + '\n'
-                udx += 1
-        return ldx, udx
-
     #
     # Replacement visitors
     #
@@ -414,16 +361,12 @@ class CtxAwareTransformer(NodeTransformer):
         self._nwith += 1
         self.generic_visit(node)
         self._nwith -= 1
-        self.insert_with_block_check(node)
         return node
 
     def visit_For(self, node):
         """Handle visiting a for statement."""
         targ = node.target
-        if isinstance(targ, (Tuple, List)):
-            self.ctxupdate(leftmostname(elt) for elt in targ.elts)
-        else:
-            self.ctxadd(leftmostname(targ))
+        self.ctxupdate(gather_names(targ))
         self.generic_visit(node)
         return node
 
@@ -431,6 +374,13 @@ class CtxAwareTransformer(NodeTransformer):
         """Handle visiting a function definition."""
         self.ctxadd(node.name)
         self.contexts.append(set())
+        args = node.args
+        argchain = [args.args, args.kwonlyargs]
+        if args.vararg is not None:
+            argchain.append((args.vararg,))
+        if args.kwarg is not None:
+            argchain.append((args.kwarg,))
+        self.ctxupdate(a.arg for a in itertools.chain.from_iterable(argchain))
         self.generic_visit(node)
         self.contexts.pop()
         return node
