@@ -15,7 +15,6 @@ import types
 import inspect
 import itertools
 import linecache
-import importlib
 import collections
 
 from xonsh.lazyasd import LazyObject
@@ -25,6 +24,7 @@ from xonsh.tools import (cast_unicode, safe_hasattr, indent,
                          print_color, format_color)
 from xonsh.platform import HAS_PYGMENTS, PYTHON_VERSION_INFO
 from xonsh.lazyimps import pygments, pyghooks
+from xonsh.style_tools import partial_color_tokenize
 
 
 # builtin docstrings to ignore
@@ -303,6 +303,7 @@ if PYTHON_VERSION_INFO < (3, 5, 0):
     FrameInfo = collections.namedtuple('FrameInfo', ['frame', 'filename',
                                                      'lineno', 'function',
                                                      'code_context', 'index'])
+
     def getouterframes(frame, context=1):
         """Wrapper for getouterframes so that it acts like the Python v3.5 version."""
         return [FrameInfo(*f) for f in inspect.getouterframes(frame, context=context)]
@@ -466,7 +467,7 @@ class Inspector(object):
                 title += '\n'
             else:
                 title += " ".ljust(title_width - title_len)
-            out += pyghooks.partial_color_tokenize(title)
+            out += partial_color_tokenize(title)
             if isinstance(content, str):
                 out[-1] = (out[-1][0], out[-1][1] + content + '\n')
             else:
@@ -630,8 +631,8 @@ class Inspector(object):
                 str_head = 'string_form'
                 if not detail_level and len(ostr) > string_max:
                     ostr = ostr[:shalf] + ' <...> ' + ostr[-shalf:]
-                    ostr = ("\n" + " " * len(str_head.expandtabs())).\
-                           join(q.strip() for q in ostr.split("\n"))
+                    ostr = ("\n" + " " * len(str_head.expandtabs())). \
+                        join(q.strip() for q in ostr.split("\n"))
                 out[str_head] = ostr
             except:  # pylint:disable=bare-except
                 pass
