@@ -13,6 +13,7 @@ import collections
 from xonsh.lazyasd import LazyObject
 from xonsh.platform import ON_DARWIN, ON_WINDOWS, ON_CYGWIN
 
+
 tasks = LazyObject(collections.deque, globals(), 'tasks')
 # Track time stamp of last exit command, so that two consecutive attempts to
 # exit can kill all jobs and exit.
@@ -69,7 +70,7 @@ if ON_WINDOWS:
     def _set_pgrp(info):
         pass
 
-    def wait_for_active_job(signal_to_send=None):
+    def wait_for_active_job(last_task=None, backgrounded=False):
         """
         Wait for the active job to finish, to be killed by SIGINT, or to be
         suspended by ctrl-z.
@@ -78,7 +79,7 @@ if ON_WINDOWS:
         active_task = get_next_task()
         # Return when there are no foreground active task
         if active_task is None:
-            return
+            return last_task
         obj = active_task['obj']
         _continue(active_task)
         while obj.returncode is None:
@@ -88,7 +89,7 @@ if ON_WINDOWS:
                 pass
             except KeyboardInterrupt:
                 _kill(active_task)
-        return wait_for_active_job()
+        return wait_for_active_job(last_task=active_task)
 
 else:
     def _continue(job):
@@ -163,7 +164,7 @@ else:
                 os.tcsetpgrp(st, pgid)
                 signal.pthread_sigmask(signal.SIG_SETMASK, oldmask)
 
-    def wait_for_active_job():
+    def wait_for_active_job(last_task=None, backgrounded=False):
         """
         Wait for the active job to finish, to be killed by SIGINT, or to be
         suspended by ctrl-z.
@@ -173,17 +174,25 @@ else:
         # Return when there are no foreground active task
         if active_task is None:
             _give_terminal_to(_shell_pgrp)  # give terminal back to the shell
-            return
+            if backgrounded:
+                # restoring sanity could probably be called whenever we return
+                # control to the shell. But it only seems to matter after a
+                # ^Z event. This *has* to be called after we give the terminal
+                # back to the shell.
+                builtins.__xonsh_shell__.shell.restore_tty_sanity()
+            return last_task
         pgrp = active_task.get('pgrp', None)
         obj = active_task['obj']
+        backgrounded = False
         # give the terminal over to the fg process
         if pgrp is not None:
             _give_terminal_to(pgrp)
         _continue(active_task)
         _, wcode = os.waitpid(obj.pid, os.WUNTRACED)
         if os.WIFSTOPPED(wcode):
-            print()  # get a newline because ^Z will have been printed
+            print('^Z')
             active_task['status'] = "stopped"
+            backgrounded = True
         elif os.WIFSIGNALED(wcode):
             print()  # get a newline because ^C will have been printed
             obj.signal = (os.WTERMSIG(wcode), os.WCOREDUMP(wcode))
@@ -191,7 +200,8 @@ else:
         else:
             obj.returncode = os.WEXITSTATUS(wcode)
             obj.signal = None
-        return wait_for_active_job()
+        return wait_for_active_job(last_task=active_task,
+                                   backgrounded=backgrounded)
 
 
 def get_next_task():
@@ -251,9 +261,7 @@ def get_next_job_number():
 
 
 def add_job(info):
-    """
-    Add a new job to the jobs dictionary.
-    """
+    """Add a new job to the jobs dictionary."""
     num = get_next_job_number()
     info['started'] = time.time()
     info['status'] = "running"
@@ -278,8 +286,9 @@ def clean_jobs():
 
         if builtins.__xonsh_all_jobs__:
             global _last_exit_time
-            if builtins.__xonsh_history__.buffer:
-                last_cmd_start = builtins.__xonsh_history__.buffer[-1]['ts'][0]
+            hist = builtins.__xonsh_history__
+            if hist is not None and len(hist.tss) > 0:
+                last_cmd_start = hist.tss[-1][0]
             else:
                 last_cmd_start = None
 
@@ -343,7 +352,6 @@ def fg(args, stdin=None):
     given as an argument, bring that job to the foreground. Additionally,
     specify "+" for the most recent job and "-" for the second most recent job.
     """
-
     _clear_dead_jobs()
     if len(tasks) == 0:
         return '', 'Cannot bring nonexistent job to foreground.\n'
@@ -374,19 +382,19 @@ def fg(args, stdin=None):
     job['bg'] = False
     job['status'] = "running"
     print_one_job(act)
+    wait_for_active_job()
 
 
 def bg(args, stdin=None):
-    """
-    xonsh command: bg
+    """xonsh command: bg
 
     Resume execution of the currently active job in the background, or, if a
     single number is given as an argument, resume that job in the background.
     """
     res = fg(args, stdin)
     if res is None:
-        curTask = get_task(tasks[0])
-        curTask['bg'] = True
-        _continue(curTask)
+        curtask = get_task(tasks[0])
+        curtask['bg'] = True
+        _continue(curtask)
     else:
         return res
