@@ -18,8 +18,19 @@ from xonsh.events import events
 import xonsh.history.main as xhm
 
 
+events.doc('on_transform_command', """
+on_transform_command(cmd: str) -> str
+
+Fired to request xontribs to transform a command line. Return the transformed
+command, or the same command if no transformaiton occurs. Only done for
+interactive sessions.
+
+This may be fired multiple times per command, with other transformers input or
+output, so design any handlers for this carefully.
+""")
+
 events.doc('on_precommand', """
-on_precommand(cmd: str) -> str
+on_precommand(cmd: str) -> None
 
 Fires just before a command is executed.
 """)
@@ -27,11 +38,11 @@ Fires just before a command is executed.
 events.doc('on_postcommand', """
 on_postcommand(cmd: str, rtn: int, out: str or None, ts: list) -> None
 
-Fires just after a command is executed.
+Fires just after a command is executed. The arguments are the same as history.
 """)
 
 
-def fire_precommand(src, show_diff=True):
+def transform_command(src, show_diff=True):
     """Returns the results of firing the precommand handles."""
     i = 0
     limit = sys.getrecursionlimit()
@@ -39,7 +50,7 @@ def fire_precommand(src, show_diff=True):
     raw = src
     while src != lst:
         lst = src
-        srcs = events.on_precommand.fire(src)
+        srcs = events.on_transform_command.fire(cmd=src)
         for s in srcs:
             if s != lst:
                 src = s
