@@ -55,26 +55,6 @@ class XonshError(Exception):
     pass
 
 
-class XonshBlockError(XonshError):
-    """Special xonsh exception for communicating the lines of block bodies."""
-
-    def __init__(self, lines, glbs, locs, *args, **kwargs):
-        """
-        Parameters
-        ----------
-        lines : list f str
-            Block lines, as if split by str.splitlines().
-        glbs : Mapping or None
-            Global execution context for lines, ie globals() of calling frame.
-        locs : Mapping or None
-            Local execution context for lines, ie locals() of calling frame.
-        """
-        super().__init__(*args, **kwargs)
-        self.lines = lines
-        self.glbs = glbs
-        self.locs = locs
-
-
 class XonshCalledProcessError(XonshError, subprocess.CalledProcessError):
     """Raised when there's an error with a called process
 
@@ -101,14 +81,33 @@ class XonshCalledProcessError(XonshError, subprocess.CalledProcessError):
         self.completed_command = completed_command
 
 
-def expandpath(path):
-    """Performs environment variable / user expansion on a given path
-    if the relevant flag has been set.
-    """
+def expand_path(s, expand_user=True):
+    """Takes a string path and expands ~ to home if expand_user is set
+    and environment vars if EXPAND_ENV_VARS is set."""
     env = getattr(builtins, '__xonsh_env__', os.environ)
     if env.get('EXPAND_ENV_VARS', False):
-        path = os.path.expanduser(expandvars(path))
-    return path
+        s = expandvars(s)
+    if expand_user:
+        # expand ~ according to Bash unquoted rules "Each variable assignment is
+        # checked for unquoted tilde-prefixes immediately following a ':' or the
+        # first '='". See the following for more details.
+        # https://www.gnu.org/software/bash/manual/html_node/Tilde-Expansion.html
+        pre, char, post = s.partition('=')
+        if char:
+            s = os.path.expanduser(pre) + char
+            s += os.pathsep.join(map(os.path.expanduser, post.split(os.pathsep)))
+        else:
+            s = os.path.expanduser(s)
+    return s
+
+
+def _expandpath(path):
+    """Performs environment variable / user expansion on a given path
+    if EXPAND_ENV_VARS is set.
+    """
+    env = getattr(builtins, '__xonsh_env__', os.environ)
+    expand_user = env.get('EXPAND_ENV_VARS', False)
+    return expand_path(path, expand_user=expand_user)
 
 
 def decode_bytes(b):
@@ -121,6 +120,20 @@ def decode_bytes(b):
     return b.decode(encoding=enc, errors=err)
 
 
+def findfirst(s, substrs):
+    """Finds whichever of the given substrings occurs first in the given string
+    and returns that substring, or returns None if no such strings occur.
+    """
+    i = len(s)
+    result = None
+    for substr in substrs:
+        pos = s.find(substr)
+        if -1 < pos < i:
+            i = pos
+            result = substr
+    return i, result
+
+
 class EnvPath(collections.MutableSequence):
     """A class that implements an environment path, which is a list of
     strings. Provides a custom method that expands all paths if the
@@ -158,9 +171,9 @@ class EnvPath(collections.MutableSequence):
     def __getitem__(self, item):
         # handle slices separately
         if isinstance(item, slice):
-            return [expandpath(i) for i in self._l[item]]
+            return [_expandpath(i) for i in self._l[item]]
         else:
-            return expandpath(self._l[item])
+            return _expandpath(self._l[item])
 
     def __setitem__(self, index, item):
         self._l.__setitem__(index, item)
@@ -559,7 +572,7 @@ def command_not_found(cmd):
 def suggest_commands(cmd, env, aliases):
     """Suggests alternative commands given an environment and aliases."""
     if not env.get('SUGGEST_COMMANDS'):
-        return
+        return ''
     thresh = env.get('SUGGEST_THRESHOLD')
     max_sugg = env.get('SUGGEST_MAX_NUM')
     if max_sugg < 0:
@@ -659,6 +672,7 @@ def is_writable_file(filepath):
     """
     Checks if a filepath is valid for writing.
     """
+    filepath = expand_path(filepath)
     # convert to absolute path if needed
     if not os.path.isabs(filepath):
         filepath = os.path.abspath(filepath)
@@ -733,16 +747,23 @@ def argvquote(arg, force=False):
         n_backslashes = 0
         cmdline = '"'
         for c in arg:
+            if c == "\\":
+                # first count the number of current backslashes
+                n_backslashes += 1
+                continue
             if c == '"':
+                # Escape all backslashes and the following double quotation mark
                 cmdline += (n_backslashes * 2 + 1) * '\\'
             else:
+                # backslashes are not special here
                 cmdline += n_backslashes * '\\'
-            if c != '\\':
-                cmdline += c
-                n_backslashes = 0
-            else:
-                n_backslashes += 1
-        return cmdline + n_backslashes * 2 * '\\' + '"'
+            n_backslashes = 0
+            cmdline += c
+        # Escape all backslashes, but let the terminating
+        # double quotation mark we add below be interpreted
+        # as a metacharacter
+        cmdline += + n_backslashes * 2 * '\\' + '"'
+        return cmdline
 
 
 def on_main_thread():
@@ -1322,7 +1343,7 @@ def _get_color_indexes(style_map):
             yield token, index, rgb
 
 
-def intensify_colors_for_cmd_exe(style_map, replace_colors=None):
+def intensify_colors_for_cmd_exe(style_map, replace_colors=None, ansi=False):
     """Returns a modified style to where colors that maps to dark
        colors are replaced with brighter versions. Also expands the
        range used by the gray colors
@@ -1334,14 +1355,26 @@ def intensify_colors_for_cmd_exe(style_map, replace_colors=None):
             (stype == 'best' and not has_prompt_toolkit())):
         return modified_style
     if replace_colors is None:
-        replace_colors = {1: '#44ffff',  # subst blue with bright cyan
-                          2: '#44ff44',  # subst green with bright green
-                          4: '#ff4444',  # subst red with bright red
-                          5: '#ff44ff',  # subst magenta with bright magenta
-                          6: '#ffff44',  # subst yellow with bright yellow
-                          9: '#00aaaa',  # subst intense blue (hard to read)
-                                         # with dark cyan (which is readable)
-                          }
+        if ansi:
+            replace_colors = {
+                1: '#ansiturquoise',  # subst blue with bright cyan
+                2: '#ansigreen',      # subst green with bright green
+                4: '#ansired',        # subst red with bright red
+                5: '#ansifuchsia',    # subst magenta with bright magenta
+                6: '#ansiyellow',     # subst yellow with bright yellow
+                9: '#ansiteal',       # subst intense blue (hard to read)
+                                      # with dark cyan (which is readable)
+            }
+        else:
+            replace_colors = {
+                1: '#44ffff',  # subst blue with bright cyan
+                2: '#44ff44',  # subst green with bright green
+                4: '#ff4444',  # subst red with bright red
+                5: '#ff44ff',  # subst magenta with bright magenta
+                6: '#ffff44',  # subst yellow with bright yellow
+                9: '#00aaaa',  # subst intense blue (hard to read)
+                               # with dark cyan (which is readable)
+            }
     for token, idx, _ in _get_color_indexes(style_map):
         if idx in replace_colors:
             modified_style[token] = replace_colors[idx]
@@ -1380,6 +1413,27 @@ def intensify_colors_on_win_setter(enable):
     return enable
 
 
+def format_std_prepost(template, env=None):
+    """Formats a template prefix/postfix string for a standard buffer.
+    Returns a string suitable for prepending or appending.
+    """
+    if not template:
+        return ''
+    env = builtins.__xonsh_env__ if env is None else env
+    shell = builtins.__xonsh_shell__.shell
+    try:
+        s = shell.prompt_formatter(template)
+    except Exception:
+        print_exception()
+    # \001\002 is there to fool pygments into not returning an empty string
+    # for potentially empty input. This happend when the template is just a
+    # color code with no visible text.
+    invis = '\001\002'
+    s = shell.format_color(invis + s + invis, force_string=True)
+    s = s.replace(invis, '')
+    return s
+
+
 _RE_STRING_START = "[bBrRuU]*"
 _RE_STRING_TRIPLE_DOUBLE = '"""'
 _RE_STRING_TRIPLE_SINGLE = "'''"
@@ -1413,8 +1467,7 @@ terminating quotes)"""
 
 
 def check_for_partial_string(x):
-    """
-    Returns the starting index (inclusive), ending index (exclusive), and
+    """Returns the starting index (inclusive), ending index (exclusive), and
     starting quote string of the most recent Python string found in the input.
 
     check_for_partial_string(x) -> (startix, endix, quote)
@@ -1642,3 +1695,45 @@ def ensure_timestamp(t, datetime_format=None):
     else:
         t = datetime.datetime.strptime(t, datetime_format).timestamp()
     return t
+
+
+def format_datetime(dt):
+    """Format datetime object to string base on $XONSH_DATETIME_FORMAT Env."""
+    format_ = builtins.__xonsh_env__['XONSH_DATETIME_FORMAT']
+    return dt.strftime(format_)
+
+
+def columnize(elems, width=80, newline='\n'):
+    """Takes an iterable of strings and returns a list of lines with the
+    elements placed in columns. Each line will be at most *width* columns.
+    The newline character will be appended to the end of each line.
+    """
+    sizes = [len(e) + 1 for e in elems]
+    nelem = len(elems)
+    ncols = 1
+    nrows = len(sizes)
+    columns = [sizes]
+    last_longest_row = max(sizes)
+    while True:
+        longest_row = sum(map(max, columns))
+        if longest_row - 1 <= width:
+            # we might be able to fit another column.
+            ncols += 1
+            nrows = nelem // ncols
+            columns = [sizes[i*nrows:(i+1)*nrows] for i in range(ncols)]
+            last_longest_row = longest_row
+        else:
+            # we can't fit another column
+            ncols -= 1
+            nrows = nelem // ncols
+            break
+    pad = (width - last_longest_row + ncols) // ncols
+    pad = pad if pad > 1 else 1
+    data = [elems[i*nrows:(i+1)*nrows] for i in range(ncols)]
+    colwidths = [max(map(len, d)) + pad for d in data]
+    colwidths[-1] -= pad
+    row_t = ''.join(['{{row[{i}]: <{{w[{i}]}}}}'.format(i=i) for i in range(ncols)])
+    row_t += newline
+    lines = [row_t.format(row=row, w=colwidths) for row in
+             itertools.zip_longest(*data, fillvalue='')]
+    return lines
