@@ -12,14 +12,15 @@ from xonsh.lazyasd import LazyObject
 from xonsh.platform import HAS_PYGMENTS
 from xonsh.tools import DefaultNotGiven, print_color, normabspath, to_bool
 from xonsh.inspectors import find_file, getouterframes
-from xonsh.environ import _replace_home
 from xonsh.lazyimps import pygments, pyghooks
+import xonsh.prompt.cwd as prompt
 
 
 terminal = LazyObject(lambda: importlib.import_module(
                                 'pygments.formatters.terminal'),
                       globals(), 'terminal')
 
+
 class TracerType(object):
     """Represents a xonsh tracer object, which keeps track of all tracing
     state. This is a singleton.
@@ -97,7 +98,8 @@ COLOR_LINE = ('{{PURPLE}}{fname}{{BLUE}}:'
 
 def tracer_format_line(fname, lineno, line, color=True, lexer=None, formatter=None):
     """Formats a trace line suitable for printing."""
-    fname = min(fname, _replace_home(fname), os.path.relpath(fname), key=len)
+    fname = min(fname, prompt._replace_home(fname), os.path.relpath(fname),
+                key=len)
     if not color:
         return COLORLESS_LINE.format(fname=fname, lineno=lineno, line=line)
     cline = COLOR_LINE.format(fname=fname, lineno=lineno)
@@ -162,7 +164,7 @@ def _color(ns, args):
 def _tracer_create_parser():
     """Creates tracer argument parser"""
     p = argparse.ArgumentParser(prog='trace',
-                       description='tool for tracing xonsh code as it runs.')
+                                description='tool for tracing xonsh code as it runs.')
     subp = p.add_subparsers(title='action', dest='action')
     onp = subp.add_parser('on', aliases=['start', 'add'],
                           help='begins tracing selected files.')
@@ -197,4 +199,3 @@ def tracermain(args=None):
     parser = _tracer_create_parser()
     ns = parser.parse_args(args)
     return _TRACER_MAIN_ACTIONS[ns.action](ns, args)
-
